# Read and parse Swahili vocabulary database
#
# 1.  Obtain your databases by going to http://www.kamusiproject.org/ and
#     clicking on the dictionaries link.
# 2.  Download the English - Swahili and Swahili - English dictionaries
#     as either ZIP or TEXT.
# 3.  Ruby swali.rb -a <eng-to-swa.text >> queue.txt
# 4.  Ruby swali.rb <swa-to-eng.text >> queue.txt
# 5.  It might be nice to make a donation to the project, for the databases.

$active = false

while ARGV.length != 0 do
	$active = (ARGV.shift == "-a")
end

SourceLang = $active ? "English Word" : "Swahili Word"
DestLang = $active ? "Swahili Word" : "English Word"
SwaliExample = "Swahili Example"
EnglishExample = "English Example"
DervLang = "Derived Language"
DervWord = "Derived Word"
PartSpeech = "Part of Speech"
NounClass = "Class"
SwaliPlural = "Swahili Plural"

def item_as_text(item)
	odd_bits = Array.new
	item_text = item[DestLang]
	item.delete(DestLang)
	[ "English Definition", "Swahili Definition", "Note" ].each {|label|
		if item.key?(label) then
			item_text += " [" + item[label] + "]"
			item.delete(label)
		end
	}
	if item.key?(SwaliPlural) then
		item_text += "\n[" + SwaliPlural + " " + item[SwaliPlural] + "]"
		item.delete(SwaliPlural)
	end
	if item.key?(SwaliExample) then
		if item.key?(EnglishExample) then
			item_text += "\n[Example: " + item[SwaliExample] + " = " + item[EnglishExample] + "]"
			item.delete(EnglishExample)
		else
			item_text += "\n[" + SwaliExample + ": " + item[SwaliExample] + "]"
		end
		item.delete(SwaliExample)
	else
		if item.key?(EnglishExample) then
			item_text += "\n[" + EnglishExample + ": " + item[EnglishExample] + "]"
			item.delete(EnglishExample)
		end
	end
	if item.key?(DervWord) then
		initial_vowel = item[DervWord] =~ /^[AEIOUaeiou]/
		item_text += "\n[derived from a" + (initial_vowel ? "n" : "") + " " + item[DervWord] + " word]"
		item.delete(DervWord)
	end
	if item.key?(DervLang) then
		item_text += "\n[derived from " + item[DervLang] + "]"
		item.delete(DervLang)
	end
	[ "Related Words", "Dialect", "Taxonomy" ].each{|label|
		if item.key?(label) then
			item_text += "\n[" + label + " " + item[label] + "]"
			item.delete(label)
		end
	}
	[ "Class", "Terminology" ].each {|label|
		if item.key?(label) then
			odd_bits << item[label]
			item.delete(label)
		end
	}
	item.each_key{|key_value|
		fail item_text + ": unused key \"" + key_value + "\""
	}
	item_text + (odd_bits.length == 0 ? "" : "\n[" + odd_bits.join(", ") + "]")
end


parsing = false

dictionary = Hash.new
dict_noun = Hash.new

item = Hash.new

last_key = nil

linecount = 0

while line = $stdin.gets
	line.sub!(/\n$/, "")
	linecount += 1
	while line =~ /\r$/ do
		line.sub!(/\r$/, " ")
		line += $stdin.gets.sub(/\n$/, "")
		linecount += 1
	end
	tag = line =~ /^\[([A-Za-z \]]*)\] (.*)$/
	if tag then
		# add tag line to list
		if item.key?($1) then
			error linecount.to_s + ": key \"" + $1 + "\" already defined"
		end
		item[$1] = $2
		last_key = $1
	elsif line =~ /^\[[A-Za-z ]*, [A-Za-z ]*\]$/ then
		# [x, y] --> throwaway
	elsif parsing then
		# time to parse and insert into dictionary 
		# As we use elements, we delete them.
		dest_word = item[SourceLang]
		fail "No Swahili at " + linecount.to_s if dest_word == nil
		part = item[PartSpeech]
		item.delete(SourceLang)
		item.delete(PartSpeech)
		item.delete("English Plural")
		fail "No " + DestLang + ": "+dest_word if not item.key?(DestLang)
		if part == "Noun" and item.key?(NounClass) then
			nounclass = item[NounClass]
			item.delete(NounClass)
			if dict_noun[nounclass] == nil then
				dict_noun[nounclass] = Hash.new
			end
			if dict_noun[nounclass][dest_word] == nil then
				dict_noun[nounclass][dest_word] = Array.new
			end
			dict_noun[nounclass][dest_word] << item_as_text(item)
		else
			if dictionary[part] == nil then
				dictionary[part] = Hash.new
			end
			if dictionary[part][dest_word] == nil then
				dictionary[part][dest_word] = Array.new
			end
			dictionary[part][dest_word] << item_as_text(item)
		end
		item = Hash.new
	end
	parsing = tag
end


def dump_class(word_class, word_list, extra_name)
	print "<category name=\"Swahili " + ($active ? "Active" : "Passive") + extra_name + " / ", word_class, "\">\n"
	word_list.each_pair{|word, defs|
		print "<q>", word, (defs.length == 1 ? "" : " ("+defs.length.to_s+")"), "</q>\n"
		print "<a>"
		if defs.length == 1 then
			print defs[0]
		else
			gap = ""
			defs.each_index{|ix|
				print gap, "(", ix+1, ") ", defs[ix]
				gap = "\n\n"
			}
		end
		print "</a>\n"
	}
	print "</category>\n"
end

dict_noun.each_pair{|noun_class, word_list|
	dump_class(noun_class, word_list, " / noun")
}

dictionary.each_pair{|word_class, word_list|
	dump_class(word_class, word_list, "")
}
